
import pygame

import data
import utils
from perks import perk

class StaticWidget(pygame.Surface):
    def __init__(self, size):
        pygame.Surface.__init__(self, size)
        self.size = size
        self.screen = pygame.display.get_surface()
        self.rect = self.get_rect()

    def set_position(self, pos):
        self.rect.topleft = pos

    def get_position(self):
        return self.rect.topleft

    def redraw(self):
        self.screen.blit(self, self.rect)
        pygame.display.update(self.rect)


class ProgressBar(StaticWidget):
    def __init__(self, size=(200,5)):
        StaticWidget.__init__(self, size)
        self.length = size[0]

    def update(self, perc):
        self.fill((128,0,0))
        pygame.draw.line(self, (255,0,0), (0, 2), (self.length*perc,2),5)
        self.redraw()



class Score(StaticWidget):
    def __init__(self, title="", digits=8, size=18):
        self.color = (183,0,0)
        fontname = "impact.ttf"
        self.font = pygame.font.Font(data.filepath(fontname),size)
        self.title = title
        self.score = 0
        self.digits = digits
        self.text = title
        self.image = self.gen_image()
        StaticWidget.__init__(self, self.image.get_size())
        self.fill((0,0,0))
        self.blit(self.gen_image(), (0,0))

            
    def gen_image(self):
        score = str(self.score)
        zeroes = "0" * (self.digits - len(score))
        msg = self.title + zeroes + score
        self.text = msg
        return self.font.render(msg, 0, self.color, (0,0,0))

    def update(self, score):
        self.score = score
        self.fill((0,0,0))
        self.blit(self.gen_image(), (0,0))
        self.redraw()

class Cursor(pygame.sprite.Sprite):
    def __init__(self, image):
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.base_image = image.copy()
        self.rect = self.image.get_rect()
        sx, sy = image.get_size()
        self.x_off = sx/2
        self.y_off = sy/2
        self.update()

    def update(self):
        x, y = pygame.mouse.get_pos()
        self.rect.topleft = (x - self.x_off, y - self.y_off)

    def set_position(self, pos):
        self.rect.topleft = pos

class AnimCursor(Cursor):
    def __init__(self, std_img, imglist):
        self.std_img = std_img
        self.imglist = imglist
        self.img_nr = 0
        self.animated = False
        #self.animated = False
        self.anim_timer = utils.Timer(1)
        Cursor.__init__(self, std_img)

    def update(self):
        if self.animated and self.anim_timer.update():
            if self.img_nr < len(self.imglist):
                img = self.imglist[self.img_nr]
                self.img_nr += 1
            else:
                img = self.std_img
                self.anmiated = False
                self.anim_timer.deactivate()
            self.image = img
        Cursor.update(self)

    def animate(self, time):
        self.anim_timer.activate()
        time = time / len(self.imglist)
        self.anim_timer.set_interval(time)
        self.animated = True
        self.img_nr = 0


class ClickArea(pygame.Rect):
    def __init__(self, area, blood_pos=(0,0), img=None, pos2=None):
        pygame.Rect.__init__(self, area)
        self.img = img
        self.deco = None
        self.deco2 = None
        if img:
            self.deco = StaticWidget(img.get_size())
            self.deco.set_position(blood_pos)

            if pos2:
                self.deco2 = StaticWidget(img.get_size())
                self.deco2.set_position(pos2)
        
        self.hover = False

    def highlight(self, bol):
        self.hover = bol
        if not self.img:
            return

        if bol:
            self.deco.blit(self.img, (0,0))
            self.deco.redraw()
            if self.deco2:
                self.deco2.blit(self.img, (0,0))
                self.deco2.redraw()
        else:
            self.deco.fill((0,0,0))
            self.deco.redraw()
            if self.deco2:
                self.deco2.fill((0,0,0))
                self.deco2.redraw()

class ToolTipArea(pygame.rect.Rect):
    def __init__(self, area, name):
        self.name = name
        pygame.rect.Rect.__init__(self, area)

class ToolTip(pygame.Surface):
    def __init__(self, size=(350,200)):
        pygame.Surface.__init__(self, size)
        self.pos = (0,0)
        self.set_colorkey((0,255,0,255))
        self.fill((0,255,0,255))
        self.active = False

    def activate(self, name, pos):
        if not self.active:
            self.active = True
            self.fill((0,255,0,255))
            text = perk[name]["desc"]
            textarea = TextArea(text)
            tx, ty = textarea.get_size()
            img = perk[name]["img_t"]
            ix, iy = img.get_size()

            border = 2
            self.blit(img, (0,0))
            self.blit(textarea, (2,iy))

            ty += iy
            points = [(0,0), (ix,0), (ix,iy), (tx,iy), (tx,ty), (0,ty)]
            pygame.draw.lines(self, (255,0,0), 1, points)

            self.pos = pos


class HighscoreText(StaticWidget):
    def __init__(self, text):
        font = data.filepath("impact.ttf")
        font = pygame.font.Font(font, 35)

        d_width = 28
        d_height = 40

        img_width = len(text)*d_width
        StaticWidget.__init__(self, (img_width, d_height))

        x = 0
        y = 0
        for c in text:
            digit = font.render(c, 0, (183,0,0), (0,0,0))
            self.blit(digit, (x,y))
            x += d_width

class TextLine(StaticWidget):
    def __init__(self, text, size=35, font="impact.ttf"):
        font = data.filepath(font)
        self.font = pygame.font.Font(font, size)
        img = self.font.render(text, 0, (183,0,0), (0,0,0))
        StaticWidget.__init__(self, img.get_size())
        self.blit(img, (0,0))

    def update(self, text):
        img = self.font.render(text, 0, (183,0,0), (0,0,0))
        self.blit(img, (0,0))
        self.redraw()


class TextEntry(StaticWidget):
    def __init__(self, text="_", fontsize=35):
        font = data.filepath("impact.ttf")
        self.font = pygame.font.Font(font, fontsize)
        img = self.font.render(text, 0, (183,0,0), (0,0,0))
        StaticWidget.__init__(self, (300,50))
        self.blit(img, (0,0))
        self.text = text
        self.full = False

    def add_char(self, char):
        if self.full:
            return

        l = len(self.text)
        if l==10:
            self.full = True
            self.text = self.text[:-1] + char
        elif l < 10:
            self.text = self.text[:-1] + char + "_"
        self.gen_image()

    def rem_char(self):
        if len(self.text) > 1:
            if not self.full:
                self.text = self.text[:-2] + "_"
            else:
                self.text = self.text[:-1] + "_"
            self.full = False
            self.gen_image()

    def update(self):
        pass

    def gen_image(self):
        self.fill((0,0,0))
        img = self.font.render(self.text, 0, (183,0,0), (0,0,0))
        self.blit(img, (0,0))
        self.redraw()


class TextArea(pygame.Surface):
    def __init__(self, text, fontsize=12, length=45):
        font = data.filepath("courier-bold")
        font = pygame.font.Font(font, fontsize)
        color = (255,255,255)
        color = (255,0,0)
        lineheight = fontsize + 4

        # generate textlines
        words = text.split(" ")
        line = ""
        lines = []
        for w in words:
            if len(line+w) <= length:
                line = line + w + " "
            else:
                lines.append(line)
                line = w + " "
        if line:
            lines.append(line)

        # generate images for textlines
        width = 0
        self.lines = []
        for line in lines:
            img = font.render(line, 1, color, (0,0,0))
            self.lines.append(img)
            w = img.get_size()[0]
            if w > width:
                width = w

        # blit imagelines
        size = (width, len(self.lines)*lineheight)
        pygame.Surface.__init__(self, size)
        self.fill((0, 0, 0))
        y = 0
        for line in self.lines:
            self.blit(line, (0, y))
            y += lineheight

class HoverHP(pygame.sprite.Sprite):
    #little hp_bar of zombies
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        self.zombie = None
        self.active = False
        size = (40,5)
        self.image = pygame.Surface(size)
        self.zhp = 0
        self.rect = self.image.get_rect()

    def activate(self, zombie):
        self.zombie = zombie
        self.active = True

    def update(self):
        if self.zombie:
            mpos = pygame.mouse.get_pos()
            self.rect.topleft = self.zombie.get_position()
            if self.zhp != self.zombie.hp:
                self.zhp = self.zombie.hp
                self.image.fill((0,0,0,255))
                width = 38 * float(self.zombie.hp) / self.zombie.hp_max
                pygame.draw.line(self.image, (255,0,0), (1,2), (width,2),3)

    def deactivate(self):
        self.kill()
        self.zombie = None
        self.active = False



