import math
import pygame
import random

import utils, data
from data import SND_MAIN, SND_ZHIT, SND_HURT
from gui import Score
from gui import ProgressBar
from perks import perk



REQ_TREE = utils.gen_req_tree(perk)
random.seed()

class Moveable(pygame.sprite.Sprite):
    def __init__(self, image):
        pygame.sprite.Sprite.__init__(self)
        self.image = image
        self.rect = self.image.get_rect()
        self.move_speed = 2
        self.speed_mod = 1.0
        self.vel = (0,0)

    def set_vel(self, vx, vy):
        vx = self.vel[0] + (vx * self.move_speed*self.speed_mod)
        vy = self.vel[1] + (vy * self.move_speed*self.speed_mod)
        self.vel = (vx, vy)

    def set_position(self, pos):
        self.rect.topleft = pos

    def get_position(self):
        return self.rect.topleft


class Bullet(Moveable):
    def __init__(self, image, to, angle, pos, speed):
        self.base_image = pygame.transform.rotate(image, angle-90)
        Moveable.__init__(self, self.base_image)
        self.set_position(pos)
        self.vel = utils.vel_towards(self.get_position(), to, speed)

        vx = self.vel[0] + random.randint(-5, 5)
        vy = self.vel[1] + random.randint(-5, 5)
        self.vel = (vx,vy)
        self.hit_flag = False
        self.hit_timer = utils.Timer(10)

        self.ticks = pygame.time.get_ticks()
        self.move_speed = 7.5

    def set_hit(self):
        self.hit_timer.activate()
        self.hit_flag = True

    def update(self):
        if self.hit_flag and self.hit_timer.update():
            self.hit_timer.deactivate()
            self.hit_flag = False

        pos = self.get_position()
        if -100 < pos[0] < 900 and -100 < pos[1] < 900:
            ticks = pygame.time.get_ticks()
            t = ticks - self.ticks
            x = pos[0] + (self.vel[0]* t / 1000.0 * self.move_speed)
            y = pos[1] + (self.vel[1]* t / 1000.0 * self.move_speed)
            self.set_position((x,y))
            self.ticks = ticks
        else:
            self.kill()  #kill if off screen

    def unpause(self):
        self.ticks = pygame.time.get_ticks()

class PowerUp(Moveable):
    """Ammo, healthpacks, whatever"""
    def __init__(self, images, pos, type):
        self.type = type
        self.images = images
        self.img_nr = 0
        self.image = self.images[self.img_nr]
        Moveable.__init__(self, self.image)
        self.set_position(pos)
        self.hover = False
        self.anim_timer = utils.Timer(40)
        self.tele_timer = utils.Timer(40)
        self.tele_timer.deactivate()
        self.timer = utils.Timer(400)


    def picked_up(self):
        self.kill()

    def update(self):
        if self.tele_timer.active and not self.hover:
            self.tele_timer.deactivate()
        if self.anim_timer.update():
            if self.img_nr:
                self.img_nr = 0
            else:
                self.img_nr = 1
            self.image = self.images[self.img_nr]
        if self.timer.update():
            self.vanish()
        self.hover = False


    def vanish(self):
        self.kill()

class People(Moveable):
    """Moveable, turnable people sprites.
    """
    def __init__(self, image):
        Moveable.__init__(self, image)
        self.angle = 0  #looking up
        self.base_image = image.copy()
        self.turn_speed = 5
        self.hp = 100.0
        self.busy = 0
        self.max_hp = 100
        self.dead = False

    def add_hp(self, points):
        self.hp += points
        if self.hp > self.max_hp:
            self.hp = self.max_hp
        if self.hp <= 0:
            self.die()

    def die(self):
        self.dead = True

    def center(self, surf):
        pos = utils.center(self.image.get_size(), surf.get_size())
        self.set_position(pos)

    def update(self):
        if not self.busy:
            pos = self.get_position()
            x = pos[0] + self.vel[0]
            y = pos[1] + self.vel[1]
            if self.angle:
                self.image = pygame.transform.rotate(self.base_image.copy(), self.angle)
            else:
                self.image = self.base_image.copy()
            self.set_position((x,y))
            self.rect.size = self.image.get_size()

    def turn(self, pos):
        """ rotate image to face pos """
        to_x, to_y = pos
        from_x, from_y = self.get_position()

        sx, sy = self.base_image.get_size()
        from_x += sx/2
        from_y += sy/2

        x_diff = to_x - from_x
        y_diff = -1* (to_y - from_y)

        c = math.sqrt(x_diff**2 + y_diff**2)

        if c == 0:
            self.angle = 0
            return

        if (x_diff > 0 and y_diff > 0) or \
          (x_diff < 0 and y_diff < 0):
            a = math.sqrt(y_diff**2)
        else:
            a = math.sqrt(x_diff**2)

        alpha = math.degrees(math.asin(1.0*a/c))

        if x_diff > 0:
            if y_diff <= 0:
                alpha -= 90
        elif x_diff < 0:
            if y_diff < 0:
                alpha -= 180
            else:
                alpha -= 270
        elif x_diff == 0:
            if y_diff >= 0:
                alpha = 90
            else:
                alpha = -90


        self.angle = alpha -90
        #print x_diff, y_diff, alpha, self.angle



class Zombie(People):
    hp_max = 5
    dam = 2

    def __init__(self, image, target):
        People.__init__(self, image)
        self.ep = random.randint(50, 250)
        x, y = 0, 0
        off_x = [-100, 900]
        off_y = [-100, 700]

        hide_at_x = random.random() < 0.5
        if hide_at_x:
            x = random.choice(off_x)
            y = random.randint(0, 600)
        else:
            x = random.randint(0, 800)
            y = random.choice(off_y)

        pos = (x, y)
        self.set_position(pos)
        self.target = target
        self.ticks = pygame.time.get_ticks()
        self.move_speed = 1.0
        self.points = random.randint(1, 10)
        self.dead = False
        self.busy = False
        self.busy_timer = utils.Timer(10)
        self.busy_timer.deactivate()
        self.stun_flag = False
        self.stun_timer = utils.Timer(10)
        self.stun_timer.deactivate()
        self.hp = self.hp_max
        self.hit_flag = False
        self.hit_timer = utils.Timer(10)
        self.hit_timer.deactivate()

    def update(self):
        if self.hit_flag:
            if self.hit_timer.update():
                self.hit_flag = False
                self.hit_timer.deactivate()

        if self.stun_flag:
            if self.stun_timer.update():
                self.stun_flag = False
                self.stun_timer.deactivate()

        if self.busy:
            if self.busy_timer.update():
                self.busy = False
                self.busy_timer.deactivate()

        if not self.busy and not self.stun_flag:
            self.turn(self.target.get_position())
            self.vel = utils.vel_towards(self.get_position(), \
                self.target.get_position(), 1.0)

        People.update(self)

    def hitting(self):
        if self.busy_timer.active or self.stun_timer.active:
            return False

        self.busy = True
        self.busy_timer.activate()
        return random.randint(self.dam, self.dam*2)

    def gets_hit(self, dam, melee=False):
        if not melee:
            self.stun(10)
        self.stun()
        self.hp -= dam
        self.hit_flag = True
        self.hit_timer.activate()
        if self.hp < 1:
            self.dead = True

    def stun(self, time=10):
        if not self.stun_flag:
            self.stun_flag = True
            self.stun_timer.set_interval(time)
            self.stun_timer.activate()


class Player(People):
    def __init__(self, image, cursor):
        People.__init__(self, image)
        self.keypress = [0,0,0,0] #up, down, left, right
        self.cursor = cursor
        self.ouch_timer = utils.Timer(20)

        # perk handling
        self.perks = []
        self.req_tree = REQ_TREE.copy()
        self.available_perks = []

        for p in self.req_tree.keys():  #req tree
            reqs = self.req_tree[p]
            if reqs==0:
                self.available_perks.append(p)
            if reqs is True:
                self.perks.append(p)

        # EP and Level
        self.ep = 0
        self.level = 1
        self.ep_levelup = 1000
        self.lvlmod = 500
        self.levelup = False
        self.ep_bonus = 1.0
        self.ep_cap = 500

        # Melee fighting
        self.slowdown = 0.5 #slowdown if hit
        self.ac = 0
        self.mdam = 0 #melee damage
        self.dodge = 0.0
        self.stun_chance = 0.0
        self.killing_blow = 0.0
        self.hitback = 0.0
        self.move_speed = 3.0

        # Special abilities
        self.ammobox = 25
        self.medkit = 20
        self.see_zhp = 0  #see zombie hp
        self.telekinesis = 0
        self.regen = 0
        self.drop_rate = 0.1

        # Shooting
        self.dam = 5
        self.burst = 5  #shoot so many bullets on click
        self.ammo = 100
        self.max_ammo = 500
        self.clipsize = 8
        self.shots = self.clipsize
        self.piercing = 0.05
        self.shooting = 0  #true if mousebuttondown
        self.cooldown = 30
        self.reload_time = 60
        self.bullet_speed = 100.0

        #timers
        self.cooldown_timer = utils.Timer(self.cooldown)
        self.cooldown_timer.deactivate()
        self.reload_timer = utils.Timer(self.reload_time)
        self.reload_timer.deactivate()
        self.regen_timer = utils.Timer(100)
        self.stun_timer = utils.Timer(20)
        self.stun_timer.deactivate()

        #widgets
        self.shot_score = Score("", 3, 17)
        self.shot_score.set_position((725, 3))  #shots
        self.shot_score.update(self.shots)

        self.ammo_score = Score("/ ", 3, 17) #ammo
        self.ammo_score.set_position((757, 3))
        self.ammo_score.update(self.ammo)

        self.ep_score = Score("", 8, 17)  #ep
        self.ep_score.set_position((50, 3))
        self.ep_score.update(0)

        self.hp_bar = ProgressBar() #hp
        self.hp_bar.set_position((300, 12))
        self.hp_bar.update(100)

        #sounds
        self.shot_snd = SND_MAIN["pumpactionshotgunfire.ogg"]
        self.reload_snd = SND_MAIN["pumpactionshotgunreload.ogg"]


    def score_draw(self):
        self.shot_score.redraw()
        self.ammo_score.redraw()
        self.ep_score.redraw()
        self.hp_bar.redraw()

#shooting
    def set_shooting(self, bol):
        if not self.busy:
            self.shooting = bol

    def shoot(self, rel_cursor):
        if self.cooldown_timer.active or self.reload_timer.active:
            return False

        if self.shots:
            self.shot_snd.play()
            self.cooldown_timer.activate()
            self.shots -= 1
            if self.shots < 0:
                self.shots = 0
            self.shot_score.update(self.shots)
            return True
        else:
            return False

    def add_hp(self, hp=None):
        if hp is None:
            hp = self.medkit
            SND_MAIN["pickingupmedkit.ogg"].play()
        People.add_hp(self, hp)
        self.hp_bar.update(float(self.hp)/self.max_hp)
        if hp < 0 and not self.ouch_timer.active:
            self.ouch_timer.activate()
            if random.random() < 0.5:
                name = random.choice(SND_HURT.keys())
                SND_HURT[name].play()
            else:
                name = random.choice(SND_ZHIT.keys())
                SND_ZHIT[name].play()

    def add_ep(self, ep):
        self.ep += int(ep * self.ep_bonus)
        self.ep_score.update(self.ep)

        #check for levelup
        if self.ep > self.ep_levelup:
            self.levelup = True
            self.lvlmod += self.ep_cap
            self.ep_levelup = self.ep_levelup + self.lvlmod
            #print self.ep_levelup
            self.level += 1

            if self.level == 43:
                self.ep_cap = 5000

    def new_perk(self, name=None):
        #select new perk, apply effects, return perk name
        self.levelup = False
        if name:
            pass
        elif not self.available_perks:
            name = "King of the Lawn"
        else:
            name = random.choice(self.available_perks)
            self.available_perks.remove(name)
            self.req_tree[name] = True
            #print name, ": ",
            for next in perk[name]["next"]:  #update tree
                self.req_tree[next] -= 1
                if self.req_tree[next] <= 0:
                    self.available_perks.append(next)
                    #print next,
            #print
        #apply effects
        new_perk = perk[name]
       #DEBUG print "==========", name
        attr = new_perk.has_key


        #adjust shotgun sound
        if name=="Semi-automatic Shotgun":
            self.shot_snd = SND_MAIN["automaticshotgunfire.ogg"]
            self.reload_snd = SND_MAIN["automaticshotgunreload.ogg"]
        elif name=="Jackhammer":
            self.shot_snd = SND_MAIN["jackhammerfire.ogg"]
            self.reload_snd = SND_MAIN["jackhammerreload1.ogg"]
        elif name=="Automatic Reloader":
            self.reload_snd = SND_MAIN["jackhammerreload2.ogg"]


        #Shotgun perks
        if attr("cooldown"):
            self.cooldown += new_perk["cooldown"]
            self.cooldown_timer.set_interval(self.cooldown)
           #DEBUG print "Cooldown: ", self.cooldown
        if attr("bulletspeed"):
            self.bullet_speed += new_perk["bulletspeed"]
           #DEBUG print "Bulletspeed: ", self.bullet_speed
        if attr("shotgundamage"):
            self.dam += new_perk["shotgundamage"]
           #DEBUG print "Damage: ", self.dam
        if attr("piercing"):
            self.piercing += new_perk["piercing"]
           #DEBUG print "Piercing: ", self.piercing
        if attr("clipsize"):
            self.clipsize += new_perk["clipsize"]
           #DEBUG print "Clipsize :", self.clipsize
        if attr("reloadtime"):
            self.reload_time += new_perk["reloadtime"]
            self.reload_timer.set_interval(self.reload_time)
           #DEBUG print "Reloadtime: ", self.reload_time
        if attr("burst"):
            self.burst += new_perk["burst"]
           #DEBUG print "Burst: ", self.burst

        #Mensch perks
        if attr("regen"):
            self.regen += new_perk["regen"]
           #DEBUG print "Regen: ", self.regen
        if attr("dmgreduction"):
            self.ac -= new_perk["dmgreduction"]
           #DEBUG print "Damage reduced: ", self.ac
        if attr("slowdownmod"):
            self.slowdown += new_perk["slowdownmod"]
           #DEBUG print "Slowdown: ", self.slowdown
        if attr("maxhp"):
            hp_perc = float(self.hp) / self.max_hp  #FIX: keep hp percentage
            self.max_hp += new_perk["maxhp"]
            self.add_hp(1)  #update hp bar
           #DEBUG print "Max HP: ", self.max_hp

        #Melee perks
        if attr("dodge"):
            self.dodge += new_perk["dodge"]
           #DEBUG print "Dodge: ", self.dodge
        if attr("hitback"):
            self.hitback += new_perk["hitback"]
           #DEBUG print "Hitback: ", self.hitback
        if attr("meleedamage"):
            self.mdam += new_perk["meleedamage"]
           #DEBUG print "Meleedamage: ", self.mdam
        if attr("medkitheal"):
            self.medkit += new_perk["medkitheal"]
           #DEBUG print "Medkit: ", self.medkit
        if attr("movement"):
            self.move_speed += new_perk["movement"]
           #DEBUG print "Movement: ", self.move_speed
        if attr("stunchance"):
            self.stun_chance += new_perk["stunchance"]
           #DEBUG print "Stunchance: ", self.stun_chance
        if attr("zombiehp"):
            self.see_zhp = True
           #DEBUG print "You see zombies' hp!"
        if attr("killingblow"):
            self.killing_blow += new_perk["killingblow"]
           #DEBUG print "Killing blow: ", self.killing_blow

        if attr("droprate"):
            self.drop_rate += new_perk["droprate"]
           #DEBUG print "Drop rate: ", self.drop_rate
        if attr("ammobonus"):
            self.ammobox += new_perk["ammobonus"]
           #DEBUG print "Ammobox: ", self.ammobox
        if attr("xpbonus"):
            self.ep_bonus += new_perk["xpbonus"]
           #DEBUG print "EP bonus", self.ep_bonus
        if attr("telekinesis"):
            self.telekinesis = True
           #DEBUG print "You have earned telekinetic abilities."
        if attr("xp"):
            ep = random.randint(new_perk["xp"],new_perk["xp"]*2)
            self.add_ep(ep)
        return name

    def add_ammo(self):
        self.ammo += self.ammobox
        if self.ammo > self.max_ammo:
            self.ammo = self.max_ammo
        self.ammo_score.update(self.ammo)
        SND_MAIN["pickingupammo.ogg"].play()

    def reload(self):
        if not self.ammo or self.shots == self.clipsize:
            return False
        elif self.ammo > self.clipsize: #reload full clip
            rel = self.clipsize - self.shots
            self.ammo -= rel
            self.shots = self.clipsize
        else:  #reload last bullets
            self.shots = self.ammo
            self.ammo = 0
        self.reload_snd.play()
        self.reload_timer.activate()
        self.cooldown_timer.deactivate()
        self.ammo_score.update(self.ammo)
        self.shot_score.update(self.shots)
        self.cursor.animate(self.reload_time)
        return True

#movement
    def set_keypress(self, nr, val):
        self.keypress[nr] = val

    def move(self):
        self.vel = (0,0)
        if self.keypress[0]:
            self.set_vel(0, -1) #move up
        if self.keypress[1]:
            self.set_vel(0, 1) #move down
        if self.keypress[2]:
            self.set_vel(-1, 0) #move left
        if self.keypress[3]:
            self.set_vel(1, 0) #move right

        #prevent player from leaving screen
        border = 10 # playing field = screen - 30px border
        scr_x, scr_y = (800,570)
        sx, sy = self.image.get_size()
        px, py = self.get_position()
        px += sx / 2
        py += sy / 2
        vx, vy = self.vel

        if px < border and vx < 0 \
            or px > scr_x-border and vx > 0:
            vx = 0
        if py < border and vy < 0 \
            or py > scr_y-border and vy > 0:
            vy = 0

        self.vel = (vx, vy)

    def update(self):
        self.move()
        self.turn(pygame.mouse.get_pos())

        if self.cooldown_timer.update():
            self.cooldown_timer.deactivate()
        if self.reload_timer.update():
            self.reload_timer.deactivate()
        if self.stun_timer.update():
            self.speed_mod = 1.0
            self.stun_timer.deactivate()
        if self.hp < self.max_hp and self.regen_timer.update():
            self.add_hp(self.regen)
        if self.ouch_timer.update():
            self.ouch_timer.deactivate()
        People.update(self)

    def stun(self):
        self.speed_mod = self.slowdown
        self.stun_timer.activate()

    def get_highscore_data(self):
        return self.ep_score.text, str(self.level)

    def die(self):
        self.hp = -1
        self.dead = True
